//
// This file is part of Nolimips, a MIPS simulator with unlimited registers
// Copyright (C) 2003, 2004, 2006, 2009 Benoit Perrot <benoit@lrde.epita.fr>
//
// Nolimips is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// Nolimips is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
#include <iostream>

#include "common.hh"

#include "task/task.hh"
#include "task/task_register.hh"

namespace task
{

  Task::Task (const std::string &option, const std::string &module_name,
	      const std::string &desc, const std::string &deps, 
	      bool needs_value):
    module_(module_name), description_(desc),
    needs_value_(needs_value)
  {
    // Compute dependencies
    std::string::size_type start = 0, end = 0;
    while (start < deps.size())
      {
	end = deps.find (' ', start);
	if (end > deps.size ())
	  end = deps.size ();
	dependencies_.push_back (deps.substr (start, end - start));
	start = end + 1;
      }

    // See if it has a short option, such as "h|help".
    if (option.size () > 1 && option[1] == '|')
      {
// 	if (key)
// 	  {
// 	    std::cerr
// 	      << program_name
// 	      << ": Task::Task (" << name
// 	      << "): both key and short option are provided"
// 	      << std::endl;
// 	    exit_set (exit_failure);
// 	  }
	short_opt_ = option.substr(0, 1);
	long_opt_ = option.substr(2);
      }
    else
      long_opt_ = option;
    
    // Register this task.
    TaskRegister::instance ().register_task (*this);
  }

  bool
  Task::set_value(const std::string & /*value */) const
  {
    // Default: fail
    return false;
  }

//   /** \brief Display dependencies of this task . */
//   void
//   Task::print_dependencies (void) const
//   {
//     std::cout << "Dependencies for task " << _name << ":" << std::endl;
//     for (deps_t::const_iterator i = _dependencies.begin();
// 	 i != _dependencies.end(); ++i)
//       std::cout << "\t" << *i << std::endl;
//   }

} // namespace task
