//
// This file is part of Nolimips, a MIPS simulator with unlimited registers
// Copyright (C) 2003, 2004, 2005 Benoit Perrot <benoit@lrde.epita.fr>
//
// Nolimips is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// Nolimips is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
#include "common.hh"

#include "inst/program.hh"

#include "vm/spim_system_library.hh"
#include "vm/nolimips_system_library.hh"
#include "vm/virtual_machine.hh"

#include "parse-tasks.hh"
#include "vm-tasks.hh"

// Vm module related tasks' implementation.
namespace vm
{

  namespace tasks
  {

    // Default system library
    static vm::SystemLibrary *system_library =
    new vm::SpimSystemLibrary(std::cin, std::cout);

    void
    select_system_library(const std::string &library_name)
    {
      if (library_name == "nolimips")
	{
	  delete system_library;
	  system_library = new vm::NolimipsSystemLibrary(std::cin,
							 std::cout, std::cerr);
	}
      else if (library_name == "spim")
	{
// 	  delete system_library;
// 	  system_library = new vm::SpimSystemLibrary(std::cin, std::cout);
	}
      else if (library_name == "none")
	{
	  delete system_library;
	  system_library = 0;
	}
      else
	{
	  std::cerr << program_name
		    << ": `" << library_name << "': no such system library"
		    << std::endl;
	  exit_set (exit_failure);
	}
    }

    void
    execute()
    {
      vm::VirtualMachine vm;
      vm.get_cpu().set_check_callee_save(check_callee_save_p);
      vm.get_cpu().set_trace(trace_exec_p);
      vm.get_cp0().set_system_library(system_library);
      vm.load_program(* parse::tasks::program);
      if (exit_status != exit_success)
	exit(exit_status);
      vm.execute();

      if (profile_p)
	{
	  std::cerr
	    << "Number of cycles: " << vm.get_cp0().get_count() << std::endl
	    << "Number of executed R-type CPU instructions: "
	    << vm.get_cpu().get_instruction_counter(inst::Inst::r_type)
	    << std::endl
	    << "Number of executed I-type CPU instructions: "
	    << vm.get_cpu().get_instruction_counter(inst::Inst::i_type)
	    << std::endl
	    << "Number of executed J-type CPU instructions: "
	    << vm.get_cpu().get_instruction_counter(inst::Inst::j_type)
	    << std::endl;	    
	}
      
      // FIXME: Quick and dirty, this should not be done here
      delete parse::tasks::program;
      parse::tasks::program = 0;
    }
    
  } // namespace tasks

} // namespace vm
