//
// This file is part of Nolimips, a MIPS simulator with unlimited registers
// Copyright (C) 2006 Benoit Perrot <benoit@lrde.epita.fr>
//
// Nolimips is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Nolimips is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
#ifndef VM_VIRTUAL_MACHINE_HXX
# define VM_VIRTUAL_MACHINE_HXX

# include "misc/contract.hh"
# include "misc/has.hh"

namespace inst { class Program; }

namespace vm
{

  //
  inline
  VirtualMachine::VirtualMachine(mode_type mode):
    mode_(mode),
    status_(stop),
    mmu_(cp0_, memory_),
    cp0_(cpu_),
    cpu_(mmu_, cp0_)
  {
  }

  //
  inline void
  VirtualMachine::write(std::ostream& ostr) const
  {
    // FIXME: print the whole cpu, and cp0.
    ostr << cpu_;
  }

  //
  inline const std::vector<register_type> &
  VirtualMachine::get_call_stack() const
  {
    return cpu_.get_call_stack();
  }

  //
  inline void
  VirtualMachine::set_runnable()
  {
    status_ = run;
    mode_ = normal;
  }

  inline void
  VirtualMachine::set_run_next()
  {
    status_ = run;
    mode_ = step;
  }

  inline VirtualMachine::status_type
  VirtualMachine::get_status() const
  {
    return status_;
  }

  //
  inline const Cp0 &
  VirtualMachine::get_cp0() const
  {
    return cp0_;
  }
  inline Cp0 &
  VirtualMachine::get_cp0()
  {
    return cp0_;
  }

  inline const Cpu &
  VirtualMachine::get_cpu() const
  {
    return cpu_;
  }
  inline Cpu &
  VirtualMachine::get_cpu()
  {
    return cpu_;
  }

  //
  inline std::ostream&
  operator<<(std::ostream &os, const VirtualMachine &vm)
  {
    vm.write(os);
    return os;
  }

} // namespace vm

#endif // !VM_VIRTUAL_MACHINE_HH
